@@ -30,7 +30,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 import numpy as np
 import scipy.sparse
-from sklearn.utils.testing import assert_array_almost_equal
+from numpy.testing import assert_array_almost_equal
 from sklearn.datasets import load_iris, load_boston
 from sklearn.linear_model import LinearRegression
 from sklearn.pipeline import make_pipeline
@@ -333,3 +333,10 @@ def test_k_fold_cv():
     )
 
     cross_val_score(clf, boston.data, boston.target, cv=KFold(n_splits=10, shuffle=True))
+
+
+def test_refit_on_new_data():
+    """Test that OneHotEncoder can refit on two data sets."""
+    ohe = OneHotEncoder()
+    ohe.fit(dense1)
+    ohe.fit(dense2)
