@@ -24,7 +24,7 @@ License along with TPOT. If not, see <http://www.gnu.org/licenses/>.
 """
 
 from tpot import TPOTClassifier, TPOTRegressor
-from tpot.base import TPOTBase
+from tpot.base import TPOTBase, _has_cuml
 from tpot.driver import float_range
 from tpot.gp_types import Output_Array
 from tpot.gp_deap import mutNodeReplacement, _wrapped_cross_val_score, pick_two_individuals_eligible_for_crossover, cxOnePoint, varOr, initialize_stats_dict
@@ -39,6 +39,8 @@ from tpot.config.regressor_mdr import tpot_mdr_regressor_config_dict
 from tpot.config.regressor_sparse import regressor_config_sparse
 from tpot.config.classifier_sparse import classifier_config_sparse
 from tpot.config.classifier_nn import classifier_config_nn
+from tpot.config.classifier_cuml import classifier_config_cuml
+from tpot.config.regressor_cuml import regressor_config_cuml
 
 import numpy as np
 import pandas as pd
@@ -193,14 +195,33 @@ def test_init_custom_parameters():
     assert tpot_obj._optimized_pipeline_score == None
     assert tpot_obj.fitted_pipeline_ == None
     assert tpot_obj._exported_pipeline_text == []
-    assert tpot_obj.log_file == sys.stdout
+    assert tpot_obj.log_file_ == sys.stdout
 
-def test_init_custom_progress_file():
+def test_init_log_file():
     """ Assert that TPOT has right file handler to save progress. """
-    file_name = "progress.txt"
+    cachedir = mkdtemp()
+    file_name = cachedir + "/progress.log"
     file_handle = open(file_name, "w")
     tpot_obj = TPOTClassifier(log_file=file_handle)
-    assert tpot_obj.log_file == file_handle
+    tpot_obj._fit_init()
+    assert tpot_obj.log_file_ == file_handle
+    file_handle.close()
+    # clean up
+    rmtree(cachedir)
+
+
+def test_init_log_file_2():
+    """ Assert that TPOT has right file handler to save progress via string input."""
+    cachedir = mkdtemp()
+    file_name = cachedir + "/progress.log"
+    tpot_obj = TPOTClassifier(log_file=file_name)
+    tpot_obj._fit_init()
+    from io import TextIOWrapper
+    assert isinstance(tpot_obj.log_file_, TextIOWrapper)
+    tpot_obj.log_file_.close()
+    # clean up
+    rmtree(cachedir)
+
 
 def test_init_default_scoring():
     """Assert that TPOT intitializes with the correct default scoring function."""
@@ -510,6 +531,15 @@ def test_conf_dict():
     tpot_obj._fit_init()
     assert tpot_obj._config_dict == regressor_config_sparse
 
+    if _has_cuml():
+        tpot_obj = TPOTClassifier(config_dict='TPOT cuML')
+        tpot_obj._fit_init()
+        assert tpot_obj._config_dict == classifier_config_cuml
+
+        tpot_obj = TPOTRegressor(config_dict='TPOT cuML')
+        tpot_obj._fit_init()
+        assert tpot_obj._config_dict == regressor_config_cuml
+
 
 def test_conf_dict_2():
     """Assert that TPOT uses a custom dictionary of operators when config_dict is Python dictionary."""
@@ -638,12 +668,9 @@ def test_score_3():
 
     # Get score from TPOT
     score = tpot_obj.score(testing_features_r, testing_target_r)
-    # On some non-amd64 systems such as arm64, a resulting score of
-    # 0.8207525232725118 was observed, so we need to add a tolerance there
-    if platform.machine() != 'amd64':
-        assert np.allclose(known_score, score, rtol=0.03)
-    else:
-        assert np.allclose(known_score, score)
+
+    assert np.allclose(known_score, score, rtol=0.03)
+
 
 
 def test_sample_weight_func():
@@ -685,18 +712,15 @@ def test_sample_weight_func():
     np.random.seed(42)
     tpot_obj.fitted_pipeline_.fit(training_features_r, training_target_r, **training_target_r_weight_dict)
     # Get score from TPOT
-    known_score = -11.586816877933911
+    known_score = -12.34832804790160
     score = tpot_obj.score(testing_features_r, testing_target_r)
 
 
     assert np.allclose(cv_score1, cv_score2)
     assert not np.allclose(cv_score1, cv_score_weight)
-    # On some non-amd64 systems such as arm64, a resulting score of
-    # 0.8207525232725118 was observed, so we need to add a tolerance there
-    if platform.machine() != 'amd64':
-        assert np.allclose(known_score, score, rtol=0.01)
-    else:
-        assert np.allclose(known_score, score)
+
+    assert np.allclose(known_score, score, rtol=0.01)
+
 
 
 def test_template_1():
@@ -912,15 +936,16 @@ def test_predict_proba_2():
 
 
 def test_predict_proba_3():
-    """Assert that the TPOT predict_proba function raises a RuntimeError when no optimized pipeline exists."""
+    """Assert that the TPOT predict_proba function raises an AttributeError when no optimized pipeline exists."""
     tpot_obj = TPOTClassifier()
     tpot_obj._fit_init()
 
-    assert_raises(RuntimeError, tpot_obj.predict_proba, testing_features)
+    with assert_raises(AttributeError) as cm:
+        tpot_obj.predict_proba(testing_features)
 
 
 def test_predict_proba_4():
-    """Assert that the TPOT predict_proba function raises a RuntimeError when the optimized pipeline do not have the predict_proba() function"""
+    """Assert that the TPOT predict_proba function raises an AttributeError when the optimized pipeline do not have the predict_proba() function"""
     tpot_obj = TPOTRegressor()
     tpot_obj._fit_init()
     pipeline_string = (
@@ -933,7 +958,9 @@ def test_predict_proba_4():
     tpot_obj.fitted_pipeline_ = tpot_obj._toolbox.compile(expr=tpot_obj._optimized_pipeline)
     tpot_obj.fitted_pipeline_.fit(training_features_r, training_target_r)
 
-    assert_raises(RuntimeError, tpot_obj.predict_proba, testing_features)
+    with assert_raises(AttributeError) as cm:
+        tpot_obj.predict_proba(testing_features)
+    
 
 
 def test_predict_proba_5():
@@ -957,6 +984,18 @@ def test_predict_proba_5():
 
     assert result.shape == (features_with_nan.shape[0], num_labels)
 
+def test_predict_proba_6():
+    """Assert that TPOT's predict_proba is exposed when available, and hidden when not."""
+
+    est = TPOTClassifier(generations=1,population_size=1, template='LogisticRegression')
+    est.fit(training_features, training_target)
+    assert hasattr(est, "predict_proba") #This model has predict_proba
+    est.predict_proba(training_features)
+
+    est = TPOTClassifier(generations=1,population_size=1, template='LinearSVC')
+    est.fit(training_features, training_target)
+    assert not hasattr(est, "predict_proba") #This model does not have predict_proba, but it hasattr shouldn't raise an error
+
 
 def test_warm_start():
     """Assert that the TPOT warm_start flag stores the pop and pareto_front from the first run."""
@@ -1110,6 +1149,40 @@ def test_fit_7():
     assert not (tpot_obj._start_datetime is None)
 
 
+def test_fit_cuml():
+    """Assert that the TPOT fit function provides an optimized pipeline when config_dict is 'TPOT cuML' if cuML is available. If not available, assert _fit_init raises a ValueError."""
+
+    tpot_clf_obj = TPOTClassifier(
+        random_state=42,
+        population_size=1,
+        offspring_size=2,
+        generations=1,
+        verbosity=0,
+        config_dict='TPOT cuML'
+    )
+
+    tpot_regr_obj = TPOTRegressor(
+        random_state=42,
+        population_size=1,
+        offspring_size=2,
+        generations=1,
+        verbosity=0,
+        config_dict='TPOT cuML'
+    )
+
+    if _has_cuml():
+        tpot_clf_obj.fit(training_features, training_target)
+        assert isinstance(tpot_clf_obj._optimized_pipeline, creator.Individual)
+        assert not (tpot_clf_obj._start_datetime is None)
+
+        tpot_regr_obj.fit(pretest_X_reg, pretest_y_reg)
+        assert isinstance(tpot_regr_obj._optimized_pipeline, creator.Individual)
+        assert not (tpot_regr_obj._start_datetime is None)
+    else:
+        assert_raises(ValueError, tpot_clf_obj._fit_init)
+        assert_raises(ValueError, tpot_regr_obj._fit_init)
+
+
 def test_memory():
     """Assert that the TPOT fit function runs normally with memory=\'auto\'."""
     tpot_obj = TPOTClassifier(
@@ -1242,7 +1315,7 @@ def test_check_periodic_pipeline():
     )
     tpot_obj.fit(training_features, training_target)
     with closing(StringIO()) as our_file:
-        tpot_obj.log_file = our_file
+        tpot_obj.log_file_ = our_file
         tpot_obj.verbosity = 3
         tpot_obj._last_pipeline_write = datetime.now()
         sleep(0.11)
@@ -1286,7 +1359,7 @@ def test_save_periodic_pipeline():
     )
     tpot_obj.fit(training_features, training_target)
     with closing(StringIO()) as our_file:
-        tpot_obj.log_file = our_file
+        tpot_obj.log_file_ = our_file
         tpot_obj.verbosity = 3
         tpot_obj._last_pipeline_write = datetime.now()
         sleep(0.11)
@@ -1316,7 +1389,7 @@ def test_save_periodic_pipeline_2():
     )
     tpot_obj.fit(training_features, training_target)
     with closing(StringIO()) as our_file:
-        tpot_obj.log_file = our_file
+        tpot_obj.log_file_ = our_file
         tpot_obj.verbosity = 3
         tpot_obj._last_pipeline_write = datetime.now()
         sleep(0.11)
@@ -1347,7 +1420,7 @@ def test_check_periodic_pipeline_3():
     )
     tpot_obj.fit(training_features, training_target)
     with closing(StringIO()) as our_file:
-        tpot_obj.log_file = our_file
+        tpot_obj.log_file_ = our_file
         tpot_obj.verbosity = 3
         tpot_obj._exported_pipeline_text = []
         tpot_obj._last_pipeline_write = datetime.now()
@@ -1590,7 +1663,7 @@ def test_update_pbar():
     # reset verbosity = 3 for checking pbar message
     tpot_obj.verbosity = 3
     with closing(StringIO()) as our_file:
-        tpot_obj.log_file=our_file
+        tpot_obj.log_file_ = our_file
         tpot_obj._pbar = tqdm(total=10, disable=False, file=our_file)
         tpot_obj._update_pbar(pbar_num=2, pbar_msg="Test Warning Message")
         our_file.seek(0)
@@ -1609,7 +1682,7 @@ def test_update_val():
     # reset verbosity = 3 for checking pbar message
     tpot_obj.verbosity = 3
     with closing(StringIO()) as our_file:
-        tpot_obj.log_file=our_file
+        tpot_obj.log_file_ = our_file
         tpot_obj._pbar = tqdm(total=10, disable=False, file=our_file)
         result_score_list = []
         result_score_list = tpot_obj._update_val(0.9999, result_score_list)
@@ -1656,7 +1729,7 @@ def test_preprocess_individuals():
     # reset verbosity = 3 for checking pbar message
     tpot_obj.verbosity = 3
     with closing(StringIO()) as our_file:
-        tpot_obj.log_file=our_file
+        tpot_obj.log_file_ = our_file
         tpot_obj._pbar = tqdm(total=2, disable=False, file=our_file)
         operator_counts, eval_individuals_str, sklearn_pipeline_list, _ = \
                                 tpot_obj._preprocess_individuals(individuals)
@@ -1702,7 +1775,7 @@ def test_preprocess_individuals_2():
     # reset verbosity = 3 for checking pbar message
     tpot_obj.verbosity = 3
     with closing(StringIO()) as our_file:
-        tpot_obj.log_file=our_file
+        tpot_obj.log_file_ = our_file
         tpot_obj._pbar = tqdm(total=3, disable=False, file=our_file)
         operator_counts, eval_individuals_str, sklearn_pipeline_list, _ = \
                                 tpot_obj._preprocess_individuals(individuals)
@@ -1749,7 +1822,7 @@ def test_preprocess_individuals_3():
     # reset verbosity = 3 for checking pbar message
 
     with closing(StringIO()) as our_file:
-        tpot_obj.log_file=our_file
+        tpot_obj.log_file_ = our_file
         tpot_obj._lambda=4
         tpot_obj._pbar = tqdm(total=2, disable=False, file=our_file)
         tpot_obj._pbar.n = 2
@@ -2406,4 +2479,4 @@ def test_clean_pipeline_string():
     ind1 = creator.Individual.from_string(with_prefix, tpot_obj._pset)
 
     pretty_string = tpot_obj.clean_pipeline_string(ind1)
-    assert pretty_string == without_prefix
+    assert pretty_string == without_prefix
\ No newline at end of file
