@@ -158,7 +158,7 @@ class OneHotEncoder(BaseEstimator, TransformerMixin):
 
         Non-categorical features are always stacked to the right of the matrix.
 
-    dtype : number type, default=np.float
+    dtype : number type, default=np.float64
         Desired dtype of output.
 
     sparse : boolean, default=True
@@ -213,7 +213,7 @@ class OneHotEncoder(BaseEstimator, TransformerMixin):
       encoding of dictionary items or strings.
     """
 
-    def __init__(self, categorical_features='auto', dtype=np.float,
+    def __init__(self, categorical_features='auto', dtype=np.float64,
                  sparse=True, minimum_fraction=None, threshold=10):
         self.categorical_features = categorical_features
         self.dtype = dtype
@@ -290,7 +290,7 @@ class OneHotEncoder(BaseEstimator, TransformerMixin):
 
         # Remember which values should not be replaced by the value 'other'
         if self.minimum_fraction is not None:
-            do_not_replace_by_other = list()
+            do_not_replace_by_other = []
             for column in range(X.shape[1]):
                 do_not_replace_by_other.append(list())
 
@@ -386,13 +386,16 @@ class OneHotEncoder(BaseEstimator, TransformerMixin):
         y: array-like {n_samples,} (Optional, ignored)
             Feature labels
         """
+
         if self.categorical_features == "auto":
-            self.categorical_features = auto_select_categorical_features(X, threshold=self.threshold)
+            self.categorical_features_ = auto_select_categorical_features(X, threshold=self.threshold)
+        else:
+            self.categorical_features_ = self.categorical_features
 
         return _transform_selected(
             X,
             self._fit_transform,
-            self.categorical_features,
+            self.categorical_features_,
             copy=True
         )
 
@@ -493,6 +496,6 @@ class OneHotEncoder(BaseEstimator, TransformerMixin):
         """
         return _transform_selected(
             X, self._transform,
-            self.categorical_features,
+            self.categorical_features_,
             copy=True
         )
